package ex1;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * Tests comparators implemented as inner classes. Testing is performed by 
 * sorting a collection of user-defined objects ({@link Person}, {@link Student}).  
 */
public class Test {


	/**
	 * Initialize the object on which to perform the test(s).\\
	 * Returns a list of Student objects.
	 * @return a List implementation containing Student objects
	 */
	public static List<Student> initTest(){

		List<Student> students = new ArrayList<Student>();

		students.add (new Student("Alice B", 10));
		students.add (new Student("Bob", 10));
		students.add (new Student("Alice E", 9));
		students.add (new Student("Trudy", 7));
		students.add (new Student("Millie", 5));	
		return students;
	}

	/**
	 * Tests the comparators implemented with regular inner classes. 
	 */
	public static void testRegular(){

		System.out.println("---------- Regular Inner classes test ----------");

		List<Student> students = initTest();

		Collections.sort(students, (new Person()).new NameComparator());

		System.out.println("Order by name:");
		for (Person s: students)
			System.out.println(s);		

		Collections.sort(students, (new Student()).new GradeComparator());

		System.out.println("\nOrder by grade:");
		for (Person s: students) System.out.println(s);

		System.out.println("------------------- End test -------------------");
	}

	/**
	 * Tests the comparators implemented using static nested classes.
	 */
	public static void testStatic(){		

		System.out.println("---------- Static nested classes test ----------");
		List<Student> students = initTest();
		Collections.sort(students, new Person.StaticNameComparator());

		System.out.println("Order by name:");
		for (Person s: students) System.out.println(s);		

		Collections.sort(students, new Student.StaticGradeComparator());

		System.out.println("\nOrder by grade:");
		for (Person s: students) System.out.println(s);
		System.out.println("------------------- End test -------------------");
	}

	/**
	 * Tests the comparators implemented using anoymous inner classes.
	 */
	public static void testAnonymous(){
		System.out.println("---------- Anonymous inner classes test ----------");
		List<Student> students = initTest();
		Collections.sort(students, new Comparator<Student>(){
			@Override
			public int compare(Student o1, Student o2) {				
				return o1.getName().compareTo(o2.getName());
			}			
		});

		System.out.println("Order by name:");
		for (Person s: students) System.out.println(s);	

		Collections.sort(students, new Comparator<Student>(){
			@Override
			public int compare(Student o1, Student o2) {				
				return -o1.getGrade().compareTo(o2.getGrade());
			}			
		});

		System.out.println("\nOrder by grade:");
		for (Person s: students) System.out.println(s);
		System.out.println("------------------- End test -------------------");		
	}

	public static void main(String[] args) {
		testRegular();
		testStatic();
		testAnonymous();
	}
}
