package ex2;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Random;

public class Test {
	
	/**
	 * ListObserver implementation for printing to stdout all information about
	 * the events, when it's notified of their occurrence.	
	 * 
	 * Static because it doesn't need anything from Test's instance
	 */
	static class PrintObserver implements ListObserver{ 
		@Override
		public void notifyEvent(ListEvent e) {
			System.out.println(e.getType() + "\telement: " + e.getElement() +
						"\t list is now: " + e.getList());					
		}
	}
	
	/**
	 * ListObserver implementation for keeping track of all events that have a 
	 * duration bigger than a threshold.
	 * 
	 * Static because it doesn't need anything from Test's instance
	 */
	
	static class DurationObserver implements ListObserver{
		
		private ArrayList<ListEvent> events;
		private int threshold;
		public DurationObserver(){
			events = new ArrayList<ListEvent>();
		}
		
		public DurationObserver(ArrayList<ListEvent> events, int threshold){
			this.events = events;
			this.threshold = threshold;
		}
		
		
		@Override
		public void notifyEvent(ListEvent e) {
			
			if (e.getDuration() > threshold) 	
				events.add(e);				
		}
	}
	
	public static void main(String args[]){
				
		Random random = new Random(0);	//fixed seed, always generates the same 
									  	//sequence of pseudo-random numbers
		DecimalFormat df = new DecimalFormat("#.##"); 
		
		/* populate the lists, no observers yet --> no events are printed */
		
		MyArrayList<Integer> list1 = new MyArrayList<Integer>();
		MyArrayList<Float> list2 = new MyArrayList<Float>();
		
		for (int i = 0; i < 10; i++){
			list1.add(random.nextInt(100)); 
			list2.add(new Float(df.format(random.nextFloat()))); 
		}
		
		System.out.println(list1);
		System.out.println(list2);
		
		/* create observers */
		
		ListObserver printObserver = new Test.PrintObserver();
		ArrayList<ListEvent> longEvents = new ArrayList<ListEvent>();
		
		// change the threshold for the durations on your machine
		DurationObserver durationObserver = new Test.DurationObserver(longEvents, 1500);
		
		/* test observers */
		
		list1.addObserver(printObserver);
		list1.addObserver(durationObserver);
		list1.add(random.nextInt(100)); 		// observer va afisa event-ul
		list2.add(new Float(df.format(random.nextFloat()))); // list2 nu are inca observeri
		
		list2.addObserver(printObserver);
		list2.addObserver(durationObserver);
		
		list1.add(0);
		list1.add(1);
		list2.add(0f);
		list2.add(1f);	
		
		list1.set(0, -100);
		list2.set(1, -100f);
		list1.set(1, -200);
		list2.set(2, -200f);
				
		list1.remove(3);
		list2.remove(3);
		list1.remove(3);
		list2.remove(3);	
				
		System.out.println("\nLong events:");
		for (ListEvent e: longEvents)
			System.out.println(e.getType() + "\telement: " + e.getElement() +
					"\t in list: " + e.getList().getClass().getName() + "@" 
					// don't print the whole list, just the instance's hashcode
					+ Integer.toHexString(e.getList().hashCode())); 
	}
}
